library linkify_text_package;

import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/widgets.dart';
import 'package:url_launcher/url_launcher.dart';
import 'dart:core';

class Word{
  final String text;
  final bool isLink;

  Word({this.text, this.isLink = false});
}

class Linkify{

  static List<Word> parseLink(String text){

    List<Word> words = [];

    RegExp exp = RegExp(
        r"((https?|ftp|smtp):\/\/)?(www.)?[a-z0-9]+(\.[a-z]+)+(\/[a-zA-Z0-9#]+\/?)*");

    Iterable<RegExpMatch> list =
    exp.allMatches(text);

    int prevEnd = 0;
    int currentStart = 0;
    int currentEnd = 0;


    list.toList().forEach((RegExpMatch e) {
      currentStart = e.start;
      currentEnd = e.end;

      if(currentStart != prevEnd){
        final String normalString = text.substring(prevEnd, currentStart);
        Word word = Word(text: normalString);
        words.add(word);

      }

      final String linkString = text.substring(currentStart, currentEnd);
      Word word = Word(text: linkString, isLink: true);
      words.add(word);

      prevEnd = currentEnd;
    });



    if(currentEnd != text.length){
      final String normalString = text.substring(currentEnd, text.length);
      Word word = Word(text: normalString);
      words.add(word);
    }

    return words;
  }




}

class Toaster {
  static showToast(final msg) {
    Fluttertoast.showToast(
      msg: msg,
      toastLength: Toast.LENGTH_LONG,
      gravity: ToastGravity.BOTTOM,
      timeInSecForIos: 1,
      backgroundColor: Colors.black,
      textColor: Colors.white,
      fontSize: 12.0,
    );
  }

  static show({
    @required String text,
    MaterialColor backgroundColor,
    MaterialColor textColor,
  }) {
    Fluttertoast.showToast(
      msg: text,
      toastLength: Toast.LENGTH_LONG,
      gravity: ToastGravity.BOTTOM,
      timeInSecForIos: 1,
      backgroundColor: backgroundColor==null?Colors.black:backgroundColor,
      textColor: textColor==null?Colors.white:textColor,
      fontSize: 12.0,
    );
  }
}




class LinkifyText extends StatefulWidget {
  final String text;
  final Color textColor;
  final double textSize;
  final Color linkColor;
  final String fontFamily;
  final bool isLinkNavigationEnable;
  final FontWeight fontWeight;
  final Color toastColor;


  LinkifyText({
    this.text,
    this.textColor,
    this.linkColor,
    this.textSize,
    this.fontFamily,
    this.fontWeight,
    this.isLinkNavigationEnable = true,
    this.toastColor,
  });

  @override
  _LinkifyTextState createState() => _LinkifyTextState();
}

class _LinkifyTextState extends State<LinkifyText> {
  List<Word> words;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();

  }

  _launchURL(String url) async {

    if(!url.startsWith(r"https?://"))
      url = "https://"+url;

    if (await canLaunch(url)) {
      await launch(url);
    } else {
      Toaster.show(text: "Sorry having issue in opening the link", backgroundColor: widget.toastColor??Colors.black);
      throw 'Could not launch $url';
    }
  }

  @override
  Widget build(BuildContext context) {
    words = Linkify.parseLink(widget.text);
    return Text.rich(
        TextSpan(children: words.map((Word word){
          return
            word.isLink ?
            TextSpan(
              text: word.text,
              style: TextStyle(
                  fontSize: widget.textSize,
                  fontFamily: widget.fontFamily,
                  fontWeight: widget.fontWeight ?? FontWeight.normal,
                  color: widget.linkColor ?? Colors.blue[700],
                  decoration: TextDecoration.underline
              ),
              recognizer: new TapGestureRecognizer()..onTap = (){
                if(!widget.isLinkNavigationEnable)
                  return;
                _launchURL(word.text);
              },
            ):
            TextSpan(
              text: word.text,
              style: TextStyle(
                  fontSize: widget.textSize ?? 15.0,
                  fontFamily: widget.fontFamily,
                  fontWeight: widget.fontWeight ?? FontWeight.normal,
                  color: widget.textColor ?? Colors.white),
            );
        }).toList())
    );
  }
}

